﻿Imports System.IO
Imports System.IO.Compression

Public Class frmMainApp

    ' Store the AppID from CumulusGate here:
    Dim strAppID As String = "SEV-SCA-001"

    ' We'll use the "Form Shown" event to start our update check and procedures:
    Private Sub Form1_Shown(sender As Object, e As EventArgs) Handles MyBase.Shown

        ' Create a new instance of the update class:
        Dim xUpdate As New cumulusgate.CumulusUpdate(strAppID)

        ' Check if a record of the app version exists, if not, create one. We'll use this to compare to the online version available
        ' (You could also store this in the app settings):
        If My.Computer.FileSystem.FileExists(Application.StartupPath() & "\currver.txt") = False Then
            IO.File.WriteAllText(Application.StartupPath() & "\currver.txt", "0.0.0")
        End If

        ' Lets make sure the 'update' folder exists, and if not, create it.
        ' Downloaded zip archives containing the update will be stored here temporarily:
        If My.Computer.FileSystem.DirectoryExists(Application.StartupPath() & "\update") = False Then
            My.Computer.FileSystem.CreateDirectory(Application.StartupPath() & "\update")
        End If

        ' We might want to update the 'update' application itself, we'll check if it was part of the download here
        ' and extract it now. We delete the archive afterwards so it does not repeat every time we start this application unless
        ' there was an update to the 'update' application:
        If My.Computer.FileSystem.FileExists(Application.StartupPath() & "\update\CU_UpdaterSample_VB.zip") Then

            Using archive As ZipArchive = ZipFile.OpenRead(Application.StartupPath() & "\update\CU_UpdaterSample_VB.zip")
                For Each entry As ZipArchiveEntry In archive.Entries
                    entry.ExtractToFile(Path.Combine(Application.StartupPath(), entry.FullName), True)
                Next
            End Using

            My.Computer.FileSystem.DeleteFile(Application.StartupPath() & "\update\CU_UpdaterSample_VB.zip")

        End If

        ' Now get the current application version, and also the online version:
        Dim strCurrVer As String = IO.File.ReadAllText(Application.StartupPath() & "\currver.txt")
        Dim strNewVer As String = xUpdate.returnCurrentWebVersion()

        ' Let's display it in the form:
        Me.Text = Me.Text & " - Ver." & strCurrVer

        ' Now we compare the versions, if they differ, then we start the prompt to update:
        If strCurrVer <> strNewVer And strNewVer <> "N/A" Then

            ' Ask the user whether to download the update:
            Dim result As DialogResult = MessageBox.Show("A newer version is available, would you like to update now?", "CumulusUpdate", MessageBoxButtons.YesNo)
            If result = DialogResult.Yes Then

                ' Let's disable certain controls so that the process does not get interrupted:
                btnClose.Enabled = False
                tabMaster.Enabled = False

                ' Download the update.
                ' We pass this form (optional) and the status label (optional) to the function which will return
                ' progress info dynamically (lblStatus in this case, which will be refreshed as the procedure progresses):
                Dim strResult As String = xUpdate.downloadUpdate(Application.StartupPath() & "\update", lblStatus, Me)

                ' If the result = 'Success' then we can proceed to process the downloaded data:
                If strResult = "Success" Then

                    ' We'll start a new process after shutting down the running app / process so we can override the contents:
                    Dim p As New ProcessStartInfo

                    ' Specify the location of the post process executable (the update application):
                    p.FileName = Application.StartupPath() & "\CU_UpdaterSample_VB.exe"

                    ' We need to pass the following arguments to our 'update' app:
                    ' Path to source zip archive (the update)
                    ' Path to destination folder, the root of this application
                    ' The process that needs to finish before the update extraction can commence.
                    ' The last parameter will ensure that whatever process is running will finish before
                    ' attempting to extract the update.

                    ' Source Zip file (obtainable from the CumulusUpdate class, because we used 'xUpdate.downloadUpdate'):
                    Dim strSourcefile As String = xUpdate.returnZipFilePath()
                    ' Destination folder to extract to:
                    Dim strDestinationFolder As String = Application.StartupPath()
                    ' Application to restart after update (could be your executable or host executable for add-ins):
                    Dim strAppToRestart As String = Application.StartupPath() & "\CU VB Sample App.exe"
                    ' Windows process to wait for before updating (could be your executable or host executable for add-ins, example "Revit.exe"):
                    Dim strProcessFinish As String = "CU VB Sample App.exe"

                    ' Add parentheses (Chr(34)) to close off spaces in path strings:
                    p.Arguments = Chr(34) & strSourcefile & Chr(34) & " " & Chr(34) & strDestinationFolder & Chr(34) & " " & Chr(34) & strAppToRestart & Chr(34) & " " & Chr(34) & strProcessFinish & Chr(34) & " " & Chr(34) & strNewVer & Chr(34)

                    ' Let's tell the user that we are waiting for something to happen:
                    Me.Text = Me.Text & " - Please wait..."

                    ' Start the process
                    Dim pxs = Process.Start(p)

                    ' Wait for the 'update' app window to appear before closing this one, this just provides reassurance
                    ' to the end-user that something is happening:
                    pxs.WaitForInputIdle()

                    ' Finally, close this app, or if this was an add-in, ask the user to close the app
                    ' (The updated will wait for the master process to finish if it was passed to the optional variable:
                    Close()

                End If

            ElseIf result = DialogResult.No Then
                ' User opted NOT to update
            End If

        End If

    End Sub





    ' The following code is just for this sample app and would not be required in your app:

    Private Sub btnClose_Click(sender As Object, e As EventArgs) Handles btnClose.Click
        Close()
    End Sub



    Sub New()

        InitializeComponent()

        PictureBox1.Image = Bitmap.FromFile(Application.StartupPath() & "\img\updatelogo_Large.png")
        PictureBox2.Image = Bitmap.FromFile(Application.StartupPath() & "\img\gatelogo_Large.png")
        PictureBox3.Image = Bitmap.FromFile(Application.StartupPath() & "\img\talklogo_Large.png")

    End Sub
End Class
