﻿Imports System.IO
Imports System.IO.Compression

Public Class frmUpdater

    ' These are the commandline arguments that we will store:
    Dim strProcName As String
    Dim strDest As String
    Dim strSrc As String
    Dim strAppPath As String
    Dim strVer As String

    Sub New()

        ' Get the commandline arguments:
        Dim args As String() = Environment.GetCommandLineArgs()

        ' This is requried by the form initializer:
        InitializeComponent()

        ' Check that commandline arguments are valid,
        ' paths should valid and there should be 6 arguments in total (5 passed by our calling app, 
        ' the first argument is a system default one, so we ignore this one):
        If My.Computer.FileSystem.FileExists(args(1)) = True And My.Computer.FileSystem.DirectoryExists(args(2)) = True And args.Count = 6 Then

            ' Store these in our variables:
            strSrc = args(1)
            strDest = args(2)
            strAppPath = args(3)
            strProcName = args(4)
            strVer = args(5)
            lblProc.Text = strProcName

            ' Update the form text so we can see what version is to be installed:
            Me.Text = Me.Text & ": Ver." & strVer

            ' This simply stores the destination folder path into the text box so we can see where it will go:
            txtDest.Text = strDest

            ' Now that we have the paths, check whether the previous process stopped completely using a timer:
            tmrProc.Start()

        Else

            ' If an error occured, notify the user:
            lblProc.Text = "All paths not valid"
            MsgBox("Something went wrong!", vbCritical, "CumulusUpdate")
            btnUpdate.Enabled = False

        End If

    End Sub

    ' This will store the process info of the process that needs to stop before we can continue:
    Dim p() As Process

    Private Sub tmrProc_Tick(sender As Object, e As EventArgs) Handles tmrProc.Tick

        ' Check processes running currently:
        p = Process.GetProcessesByName(IO.Path.GetFileNameWithoutExtension(strProcName))
        ' If it is still running, suspend the update button and features:
        If p.Count > 0 Then
            btnUpdate.Enabled = False
        Else
            ' The process has stopped, so enable the 'Update' button:
            tmrProc.Stop()
            ' Update this label to show the process has finished:
            lblProc.Text = strProcName & "-Finished"
            ' Enable the 'Update' button:
            btnUpdate.Enabled = True
        End If

    End Sub



    Private Sub btnUpdate_Click(sender As Object, e As EventArgs) Handles btnUpdate.Click

        ' Disable the 'Update' button to avoid multiple attempts to run this function simultaneously:
        btnUpdate.Enabled = False

        ' Make sure to reference System.IO.Compression and also System.IO.Compression.FileSystem:

        ' We first check whether the relevant folder paths exist:
        Using archive As ZipArchive = ZipFile.OpenRead(strSrc)
            For Each entry As ZipArchiveEntry In archive.Entries
                ' If the folder paths do not exist, then create them:
                If My.Computer.FileSystem.DirectoryExists(IO.Path.GetDirectoryName(Path.Combine(strDest, entry.FullName))) = False Then
                    My.Computer.FileSystem.CreateDirectory(IO.Path.GetDirectoryName(Path.Combine(strDest, entry.FullName)))
                End If
                ' Now extract the file to the destination path:
                entry.ExtractToFile(Path.Combine(strDest, entry.FullName), True)
            Next
        End Using

        ' Create a new process to re-launch our main app:
        Dim pxs As Process

        ' If the main app path is valid, then restart it:
        If strAppPath <> "N/A" Then
            ' Tell the user what's about to happen:
            MsgBox("Done!" & vbNewLine & "Restarting Application: " & IO.Path.GetFileNameWithoutExtension(strAppPath))

            ' Start the process but also update the current version text file:
            pxs = Process.Start(strAppPath)
            If IO.Path.GetDirectoryName(Application.StartupPath()) = IO.Path.GetDirectoryName(strDest) Then
                IO.File.WriteAllText(Application.StartupPath() & "\currver.txt", strVer)
            End If

            ' Wait for the main app to show before closing this app:
            pxs.WaitForInputIdle()
        End If

        ' Finally, close the 'update' app:
        Close()

    End Sub



End Class
